@@ -6,18 +6,17 @@ import asyncio
 import io
 import pathlib
 import random
-from typing import Iterable
+from typing import Any, Iterable
 
 import discord
 import flanautils
 from discord.ext.commands import Bot
-from discord.ui import Button, View
 from flanautils import Media, MediaType, NotFoundError, OrderedSet, return_if_first_empty
 
 from multibot import constants
 from multibot.bots.multi_bot import MultiBot, parse_arguments
 from multibot.exceptions import LimitError, SendError, UserDisconnectedError
-from multibot.models import Chat, Message, Mute, Platform, Role, User
+from multibot.models import Button, ButtonsInfo, Chat, Message, Mute, Platform, Role, User
 
 
 # --------------------------------------------------------------------------------------------------- #
@@ -36,7 +35,10 @@ class DiscordBot(MultiBot[Bot]):
             case Message():
                 event = event.original_event
 
-        await event.response.defer()
+        try:
+            await event.response.defer()
+        except AttributeError:
+            pass
 
     def _add_handlers(self):
         super()._add_handlers()
@@ -106,7 +108,7 @@ class DiscordBot(MultiBot[Bot]):
                         return button.label
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def _get_button_pressed_user(self, event: constants.DISCORD_EVENT) -> User | None:
+    async def _get_button_presser_user(self, event: constants.DISCORD_EVENT) -> User | None:
         try:
             return self._create_user_from_discord_user(event.user)
         except AttributeError:
@@ -137,6 +139,7 @@ class DiscordBot(MultiBot[Bot]):
         chat = await self._get_chat(original_message)
         if chat.original_object.guild:
             text = await self._get_text(original_message)
+            text = flanautils.remove_symbols(text, replace_with=' ')
             words = text.lower().split()
             for member in chat.original_object.guild.members:
                 user_name = f'{member.name}#{member.discriminator}'.lower()
@@ -313,7 +316,7 @@ class DiscordBot(MultiBot[Bot]):
         return users
 
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
-    async def get_chat(self, chat: int | str | User | Chat | Message = None) -> Chat | None:
+    async def get_chat(self, chat: int | str | User | Chat | Message) -> Chat | None:
         match chat:
             case int(chat_id):
                 pass
@@ -339,6 +342,21 @@ class DiscordBot(MultiBot[Bot]):
         else:
             return await self.get_user(user, group_)
 
+    @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
+    async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
+        match message:
+            case int(message_id):
+                pass
+            case str(message_id):
+                message_id = int(message_id)
+            case Message():
+                return message
+            case _:
+                raise TypeError('bad arguments')
+
+        chat = await self.get_chat(chat)
+        return await self._get_message(await chat.original_object.fetch_message(message_id))
+
     @return_if_first_empty(exclude_self_types='DiscordBot', globals_=globals())
     async def get_roles(self, group_: int | str | Chat | Message) -> list[Role]:
         if not (discord_group := await self._get_discord_group(group_)):
@@ -416,65 +434,86 @@ class DiscordBot(MultiBot[Bot]):
     @parse_arguments
     async def send(
         self,
-        text='',
+        text: str = None,
         media: Media = None,
-        buttons: list[str | list[str]] | None = None,
+        buttons: list[str | tuple[str, bool] | Button | list[str | tuple[str, bool] | Button]] | None = None,
         chat: int | str | User | Chat | Message | None = None,
         message: Message = None,
         *,
+        buttons_key: Any = None,
         reply_to: int | str | Message = None,
+        contents: dict = None,
         silent: bool = False,
         send_as_file: bool = None,
         edit=False
     ) -> Message | None:
-        def create_view() -> View | None:
-            if not buttons:
-                return
-
-            view_ = View(timeout=None)
-            for i, row in enumerate(buttons):
-                for button_text in row:
-                    discord_button = Button(label=button_text, row=i)
-                    discord_button.callback = self._on_button_press_raw
-                    view_.add_item(discord_button)
-
-            return view_
-
         text = self._parse_html_to_discord_markdown(text)
         file = await self._prepare_media_to_send(media)
+        view = None
+
+        if buttons:
+            view = discord.ui.View(timeout=None)
+            for i, row in enumerate(buttons):
+                for button in row:
+                    discord_button = discord.ui.Button(label=button.text, row=i)
+                    discord_button.callback = self._on_button_press_raw
+                    view.add_item(discord_button)
 
         if edit:
             kwargs = {}
+            if text is not None:
+                kwargs['content'] = text
             if file:
                 kwargs['attachments'] = [file]
             if buttons is not None:
-                kwargs['view'] = create_view()
-            message.original_object = await message.original_object.edit(content=text, **kwargs)
-            return message
-        else:
-            match reply_to:
-                case int(message_id):
-                    reply_to = await chat.original_object.fetch_message(message_id)
-                case str(message_id):
-                    reply_to = await chat.original_object.fetch_message(int(message_id))
-                case Message() as message_to_reply:
-                    reply_to = message_to_reply.original_object
+                kwargs['view'] = view
+                message.buttons_info.buttons = buttons
+            if buttons_key is not None:
+                message.buttons_info.key = buttons_key
 
-            try:
-                bot_message = await self._get_message(await chat.original_object.send(text, file=file, view=create_view(), reference=reply_to))
-            except discord.errors.HTTPException as e:
-                if 'too large' in str(e).lower():
-                    if random.randint(0, 10):
-                        error_message = 'El archivo pesa más de 8 MB.'
-                    else:
-                        error_message = 'El archivo pesa mas que tu madre'
-                    await self._manage_exceptions(SendError(error_message), chat)
-                    return
-                raise e
-            if content := getattr(media, 'content', None):
-                bot_message.contents = [content]
-            bot_message.save()
-            return bot_message
+            message.original_object = await message.original_object.edit(**kwargs)
+            last_contents = message.contents
+            if media_content := getattr(media, 'content', None):
+                del last_contents['media']
+            message.contents = {'media': media_content}
+            if contents is None:
+                message.contents |= last_contents
+            else:
+                message.contents |= contents
+
+            message.update_last_edit()
+            message.save()
+
+            return message
+
+        match reply_to:
+            case int(message_id):
+                reply_to = await chat.original_object.fetch_message(message_id)
+            case str(message_id):
+                reply_to = await chat.original_object.fetch_message(int(message_id))
+            case Message() as message_to_reply:
+                reply_to = message_to_reply.original_object
+
+        try:
+            bot_message = await self._get_message(await chat.original_object.send(text, file=file, view=view, reference=reply_to))
+        except discord.errors.HTTPException as e:
+            if 'too large' in str(e).lower():
+                if random.randint(0, 10):
+                    error_message = 'El archivo pesa más de 8 MB.'
+                else:
+                    error_message = 'El archivo pesa mas que tu madre'
+                await self._manage_exceptions(SendError(error_message), chat)
+                return
+            raise e
+
+        bot_message.buttons_info = ButtonsInfo(buttons=buttons, key=buttons_key)
+        bot_message.contents = {'media': getattr(media, 'content', None)}
+        if contents is not None:
+            bot_message.contents |= contents
+
+        bot_message.save()
+
+        return bot_message
 
     def start(self):
         async def start_():
