@@ -28,7 +28,7 @@ from flanautils import AmbiguityError, Media, NotFoundError, OrderedSet, RatioMa
 
 from multibot import constants
 from multibot.exceptions import LimitError, SendError
-from multibot.models import Ban, BotAction, Chat, Message, Mute, Platform, RegisteredCallback, Role, User
+from multibot.models import Ban, BotAction, Button, ButtonsGroup, Chat, Message, Mute, Platform, RegisteredButtonCallback, RegisteredCallback, Role, User
 
 
 # ---------------------------------------------------------- #
@@ -60,10 +60,11 @@ def admin(func_: Callable = None, /, is_=True, send_negative=False) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+        async def wrapper(self: MultiBot, message: Message):
             message = message
             if is_ == message.author.is_admin or message.chat.is_private:
-                return await func(self, message, *args, **kwargs)
+                return await func(self, message)
+            await self._accept_button_event(message)
             if send_negative:
                 await self.send_negative(message)
 
@@ -74,7 +75,9 @@ def admin(func_: Callable = None, /, is_=True, send_negative=False) -> Callable:
 
 def block(func: Callable) -> Callable:
     @functools.wraps(func)
-    async def wrapper(*_args, **_kwargs):
+    @find_message
+    async def wrapper(self: MultiBot, message: Message):
+        await self._accept_button_event(message)
         return
 
     return wrapper
@@ -85,9 +88,10 @@ def bot_mentioned(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+        async def wrapper(self: MultiBot, message: Message):
             if is_ == self.is_bot_mentioned(message):
-                return await func(self, message, *args, **kwargs)
+                return await func(self, message)
+            await self._accept_button_event(message)
 
         return wrapper
 
@@ -99,9 +103,10 @@ def group(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+        async def wrapper(self: MultiBot, message: Message):
             if is_ == message.chat.is_group:
-                return await func(self, message, *args, **kwargs)
+                return await func(self, message)
+            await self._accept_button_event(message)
 
         return wrapper
 
@@ -111,9 +116,10 @@ def group(func_: Callable = None, /, is_=True) -> Callable:
 def ignore_self_message(func: Callable) -> Callable:
     @functools.wraps(func)
     @find_message
-    async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+    async def wrapper(self: MultiBot, message: Message):
         if message.author.id != self.id:
-            return await func(self, message, *args, **kwargs)
+            return await func(self, message)
+        await self._accept_button_event(message)
 
     return wrapper
 
@@ -123,9 +129,10 @@ def inline(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+        async def wrapper(self: MultiBot, message: Message):
             if message.is_inline is None or is_ == message.is_inline:
-                return await func(self, message, *args, **kwargs)
+                return await func(self, message)
+            await self._accept_button_event(message)
 
         return wrapper
 
@@ -135,9 +142,10 @@ def inline(func_: Callable = None, /, is_=True) -> Callable:
 def out_of_service(func: Callable) -> Callable:
     @functools.wraps(func)
     @find_message
-    async def wrapper(self: MultiBot, message: Message, *_args, **_kwargs):
+    async def wrapper(self: MultiBot, message: Message):
         if self.is_bot_mentioned(message) or message.chat.is_private:
             await self.send(random.choice(constants.OUT_OF_SERVICES_PHRASES), message)
+        await self._accept_button_event(message)
 
     return wrapper
 
@@ -145,10 +153,36 @@ def out_of_service(func: Callable) -> Callable:
 def parse_arguments(func: Callable) -> Callable:
     @functools.wraps(func)
     async def wrapper(*args, **kwargs) -> Any:
+        def parse_buttons(buttons_) -> list[list[Button]] | None:
+            match buttons_:
+                case [str(), *_] as buttons_:
+                    buttons_ = [[Button(button_text, False) for button_text in buttons_]]
+                case [(str(), bool()), *_] as buttons_:
+                    buttons_ = [[Button(button_text, is_checked) for button_text, is_checked in buttons_]]
+                case [Button(), *_] as buttons_:
+                    buttons_ = [list(buttons_)]
+                case [[str(), *_], *_] as buttons_:
+                    buttons_ = list(buttons_)
+                    for i, buttons_row in enumerate(buttons_):
+                        buttons_[i] = [Button(button_text, False) for button_text in buttons_row]
+                case [[(str(), bool()), *_], *_] as buttons_:
+                    buttons_ = list(buttons_)
+                    for i, buttons_row in enumerate(buttons_):
+                        buttons_[i] = [Button(button_text, is_checked) for button_text, is_checked in buttons_row]
+                case [[Button(), *_], *_] as buttons_:
+                    buttons_ = list(buttons_)
+                    for i, buttons_row in enumerate(buttons_):
+                        buttons_[i] = list(buttons_row)
+                case [] as buttons_:
+                    pass
+                case _:
+                    return
+            return buttons_
+
         self: MultiBot | None = None
-        text = ''
+        text: str | None = None
         media: Media | None = None
-        buttons: list[str | list[str]] | None = None
+        buttons: list[str | tuple[str, bool] | Button | list[str | tuple[str, bool] | Button]] | None = None
         chat: int | str | User | Chat | Message | None = None
         message: Message | None = None
 
@@ -162,22 +196,20 @@ def parse_arguments(func: Callable) -> Callable:
                     text = str(number)
                 case Media() as media:
                     pass
-                case [str(), *_] as buttons:
-                    buttons = [list(buttons)]
-                case [[str(), *_], *_] as buttons:
-                    buttons = list(buttons)
-                    for i, buttons_row in enumerate(buttons):
-                        buttons[i] = list(buttons_row)
                 case User() as user:
                     chat = user
                 case Chat() as chat:
                     pass
                 case Message() as message:
                     pass
+                case _:
+                    buttons = parse_buttons(arg)
 
         chat = await self.get_chat(kwargs.get('chat', chat))
-        reply_to = kwargs.get('reply_to', None)
-        edit = kwargs.get('edit', None)
+        if 'buttons' in kwargs:
+            buttons = parse_buttons(kwargs['buttons'])
+        reply_to = kwargs.get('reply_to')
+        edit = kwargs.get('edit')
 
         if reply_to is not None:
             if chat:
@@ -194,9 +226,10 @@ def parse_arguments(func: Callable) -> Callable:
         if not message and isinstance(reply_to, Message):
             message = reply_to
 
-        for arg_name in ('self', 'text', 'media', 'buttons', 'message'):
+        for arg_name in ('self', 'text', 'media', 'message'):
             if arg_name not in kwargs:
                 kwargs[arg_name] = locals()[arg_name]
+        kwargs['buttons'] = buttons
         kwargs['chat'] = chat
 
         return await func(**kwargs)
@@ -209,9 +242,10 @@ def reply(func_: Callable = None, /, is_=True) -> Callable:
     def decorator(func: Callable) -> Callable:
         @functools.wraps(func)
         @find_message
-        async def wrapper(self: MultiBot, message: Message, *args, **kwargs):
+        async def wrapper(self: MultiBot, message: Message):
             if is_ == bool(message.replied_message):
-                return await func(self, message, *args, **kwargs)
+                return await func(self, message)
+            await self._accept_button_event(message)
 
         return wrapper
 
@@ -239,7 +273,7 @@ class MultiBot(Generic[T], ABC):
         self.token: str = bot_token
         self.client: T = bot_client
         self._registered_callbacks: list[RegisteredCallback] = []
-        self._registered_button_callbacks: list[RegisteredCallback] = []
+        self._registered_button_callbacks: list[RegisteredButtonCallback] = []
 
         self._add_handlers()
 
@@ -268,7 +302,7 @@ class MultiBot(Generic[T], ABC):
 
         self.register(self._on_users, constants.KEYWORDS['user'])
 
-        self.register_button(self._on_users_button_press, always=True)
+        self.register_button(self._on_users_button_press, ButtonsGroup.USERS)
 
     async def _ban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
@@ -306,7 +340,7 @@ class MultiBot(Generic[T], ABC):
         pass
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
-    async def _get_button_pressed_user(self, event: constants.MESSAGE_EVENT) -> User | None:
+    async def _get_button_presser_user(self, event: constants.MESSAGE_EVENT) -> User | None:
         pass
 
     @abstractmethod
@@ -328,8 +362,6 @@ class MultiBot(Generic[T], ABC):
             id=await self._get_message_id(original_message),
             author=await self._get_author(original_message),
             text=await self._get_text(original_message),
-            button_pressed_text=await self._get_button_pressed_text(event),
-            button_pressed_user=await self._get_button_pressed_user(event),
             mentions=await self._get_mentions(original_message),
             chat=await self._get_chat(original_message),
             replied_message=await self._get_replied_message(original_message),
@@ -337,6 +369,11 @@ class MultiBot(Generic[T], ABC):
             original_object=original_message,
             original_event=event
         )
+        message.resolve()
+        message.pull_from_database()
+        if message.buttons_info:
+            message.buttons_info.pressed_text = await self._get_button_pressed_text(event)
+            message.buttons_info.presser_user = await self._get_button_presser_user(event)
         message.save(pull_overwrite_fields=('_id', 'config'))
 
         return message
@@ -392,9 +429,11 @@ class MultiBot(Generic[T], ABC):
         minimum_ratio_to_match: float = constants.MINIMUM_RATIO_TO_MATCH
     ) -> OrderedSet[RegisteredCallback]:
         text = text.lower()
-        # text = flanautils.remove_accents(text)
+        text = flanautils.replace(text, {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u',
+                                         'à': 'a', 'è': 'e', 'ì': 'i', 'ò': 'o', 'ù': 'u',
+                                         'â': 'a', 'ê': 'e', 'î': 'i', 'ô': 'o', 'û': 'u',
+                                         'ä': 'a', 'ë': 'e', 'ï': 'i', 'ö': 'o', 'ü': 'u'})
         text = flanautils.translate(text, {'?': ' ', '¿': ' ', '!': ' ', '¡': ' '})
-        # text = flanautils.translate(text, {'auto': 'automatico', 'matico': None, 'matic': None})
         original_text_words = OrderedSet(text.split())
         text_words = original_text_words - flanautils.CommonWords.all_words
 
@@ -410,7 +449,7 @@ class MultiBot(Generic[T], ABC):
                 mached_keywords_groups = 0
                 total_ratio = 0
                 for keywords_group in registered_callback.keywords:
-                    text_words += [original_text_word for original_text_word in original_text_words if original_text_word in keywords_group]
+                    text_words += [original_text_word for original_text_word in original_text_words if flanautils.cartesian_product_string_matching(original_text_word, keywords_group, min_ratio=registered_callback.min_ratio)]
                     word_matches = flanautils.cartesian_product_string_matching(text_words, keywords_group, min_ratio=registered_callback.min_ratio)
                     ratio = sum((max(matches.values()) + 1) ** ratio_reward_exponent for matches in word_matches.values())
                     try:
@@ -461,12 +500,11 @@ class MultiBot(Generic[T], ABC):
 
     @find_message
     async def _on_button_press_raw(self, message: Message):
-        try:
-            registered_callbacks = self._parse_callbacks(message.button_pressed_text, self._registered_button_callbacks)
-        except AmbiguityError as e:
-            await self._manage_exceptions(e, message)
-        else:
-            for registered_callback in registered_callbacks:
+        if getattr(message.buttons_info, 'key', None) is None:
+            return
+
+        for registered_callback in self._registered_button_callbacks:
+            if registered_callback.key == message.buttons_info.key:
                 await registered_callback(message)
 
     @inline(False)
@@ -533,49 +571,34 @@ class MultiBot(Generic[T], ABC):
         user_names = [f'<@{user.id}>' for user in await self.find_users_by_roles([], message)]
         joined_user_names = ', '.join(user_names)
         await self.send(
-            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:<b>\n"
+            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:</b>\n"
             f"{joined_user_names}\n\n"
-            f"<b>Filtrar usuarios por roles:<b>",
+            f"<b>Filtrar usuarios por roles:</b>",
             flanautils.chunks([f'❌  {role_name}' for role_name in role_names], 5),
-            message
+            message,
+            buttons_key=ButtonsGroup.USERS
         )
 
     async def _on_users_button_press(self, message: Message):
-        try:
-            button_role_name = message.button_pressed_text.split(maxsplit=1)[1]
-        except IndexError:
-            return
-        if not (role_names := [role.name for role in await self.get_roles(message.chat)]) or button_role_name not in role_names:
-            return
-
         await self._accept_button_event(message)
 
-        new_buttons = []
-        selected_role_names = []
-        for row in message.original_object.components:
-            new_row = []
-            for button in row.children:
-                emoji, role_name = button.label.split(maxsplit=1)
-                if role_name == button_role_name:
-                    if emoji == '✔':
-                        new_emoji = '❌'
-                    else:
-                        new_emoji = '✔'
-                        selected_role_names.append(role_name)
-                    new_row.append(f'{new_emoji}  {role_name}')
-                else:
-                    if emoji == '✔':
-                        selected_role_names.append(role_name)
-                    new_row.append(button.label)
-            new_buttons.append(new_row)
+        try:
+            button_role_name = message.buttons_info.pressed_text.split(maxsplit=1)[1]
+        except IndexError:
+            return
 
-        await self.edit(new_buttons, message)
+        pressed_button = message.buttons_info[message.buttons_info.pressed_text]
+        pressed_button.is_checked = not pressed_button.is_checked
+        pressed_button.text = f"{'✔' if pressed_button.is_checked else '❌'}  {button_role_name}"
+
+        selected_role_names = [checked_button.text.split(maxsplit=1)[1] for checked_button in message.buttons_info.checked_buttons()]
         user_names = [f'<@{user.id}>' for user in await self.find_users_by_roles(selected_role_names, message)]
         joined_user_names = ', '.join(user_names)
         await self.edit(
-            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:<b>\n"
+            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:</b>\n"
             f"{joined_user_names}\n\n"
-            f"<b>Filtrar usuarios por roles:<b>",
+            f"<b>Filtrar usuarios por roles:</b>",
+            message.buttons_info.buttons,
             message
         )
 
@@ -640,6 +663,10 @@ class MultiBot(Generic[T], ABC):
     async def get_me(self, group_: int | str | Chat | Message = None) -> User | None:
         pass
 
+    @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
+    async def get_message(self, chat: int | str | User | Chat | Message, message: int | str | Message) -> Message | None:
+        pass
+
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def get_role(self, role: int | str | Role, group_: int | str | Chat | Message) -> Role | None:
         if isinstance(role, Role):
@@ -729,24 +756,38 @@ class MultiBot(Generic[T], ABC):
         return decorator(func_) if func_ else decorator
 
     @overload
-    def register_button(self, func_: Callable = None, keywords=(), min_ratio=constants.PARSE_BUTTON_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register_button(self, func_: Callable = None, key: Any = None):
         pass
 
     @overload
-    def register_button(self, keywords=(), min_ratio=constants.PARSE_BUTTON_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register_button(self, key: Any = None):
         pass
 
     @shift_args_if_called(exclude_self_types='MultiBot', globals_=globals())
-    def register_button(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), min_ratio=constants.PARSE_BUTTON_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register_button(self, func_: Callable = None, key: Any = None):
         def decorator(func):
-            self._registered_button_callbacks.append(RegisteredCallback(func, keywords, min_ratio, always, default))
+            self._registered_button_callbacks.append(RegisteredButtonCallback(func, key))
             return func
 
         return decorator(func_) if func_ else decorator
 
     @abstractmethod
     @parse_arguments
-    async def send(self, text='', media: Media = None, buttons: list[str | list[str]] | None = None, chat: int | str | User | Chat | Message | None = None, message: Message = None, *, reply_to: int | str | Message = None, silent: bool = False, send_as_file: bool = None, edit=False) -> Message | None:
+    async def send(
+        self,
+        text='',
+        media: Media = None,
+        buttons: list[str | tuple[str, bool] | list[str | tuple[str, bool]]] | None = None,
+        chat: int | str | User | Chat | Message | None = None,
+        message: Message = None,
+        *,
+        buttons_key: Any = None,
+        reply_to: int | str | Message = None,
+        contents: dict = None,
+        silent: bool = False,
+        send_as_file: bool = None,
+        edit=False
+    ) -> Message | None:
         pass
 
     @parse_arguments
